/* pgmcload.c : load PGM data from a C variable
**
** Copyright (C) 1994 by Michael J. Gourlay
**
** NO WARRANTEES, EXPRESS OR IMPLIED.
*/

/* ---------------------------------------------------------------------- */
#include "showtime.h"
#include "pgmcload.h"
#include "video.h"

/* ---------------------------------------------------------------------- */

/*
** load_pgm_from_c : take a c char array and turn it into an image
*/
void
load_pgm_from_c(int cols, int rows, screendata_t *sdP, unsigned char *pgm_img)
{
  unsigned int row;
  char         *mr;
  long         bytes_needed;
  long         pgmic; /* pgm index */

  /* Initialize sdP if necessary */
  if((rows!=sdP->num_rows)||(cols!=sdP->num_cols))
  {
    free_movie(sdP);
    if(init_movie(rows, cols, sdP)) {
      logmesg("\nload_pgm_from_c: failed to init_movie\n");
      bye_bye(2);
    }
  }

  /* Allocate memory if necessary */
  for(row=0; row<rows; row++) {

    bytes_needed = MOVIE_IMAGE_SIZE(0, 0, cols-1, 0);
    sdP->header_size = bytes_needed-cols;

    /* Allocate memory for the row, but only if necessary */
    if(sdP->data[row] == NULL) {
      sdP->data[row] = (unsigned char *)malloc((size_t)bytes_needed);
      if(sdP->data[row] == NULL) {
        logmesg("load_pgm_from_c: out of memory at row %i.\n", row);
        reset_display();
        bye_bye(3);
      }
    }

    /* Set the image header */
#if defined(MSC)
    _getimage(0, 0, cols-1, 0, sdP->data[row]);
#elif defined(TC)
    ((UWORD *)sdP->data[row])[0] = (UWORD)(cols*COL_FACTOR);
    ((UWORD *)sdP->data[row])[1] = (UWORD)(1);
#endif /* architecture */
  }

  /* Load each portion of the PGM image */
  pgmic=0; /* pgm image counter */
  for(row=0; row<rows; row++) {
    /* Read the image data */
    mr=memcpy((void *)(sdP->data[row]+sdP->header_size), &(pgm_img[pgmic]),
              (size_t)cols);
    if(mr==NULL) {
      logmesg("\nmemcpy failed in l_p_f_c at row %i\n", row);
      bye_bye(3);
    }
    pgmic+=cols;
  }
}
